(defvar ldap-mode-syntax-table nil)
(defvar ldap-mode-abbrev-table nil "*")
(define-abbrev-table 'ldap-mode-abbrev-table ())
(defvar ldap-mode-hook nil "*")
(defvar ldap-indent-level 2)
(defvar ldap-keyword-face 'ldap-keyword-face)

(if (not (facep ldap-keyword-face))
    (copy-face 'underline ldap-keyword-face))

(if ldap-mode-syntax-table
    ()
  (setq ldap-mode-syntax-table (make-syntax-table))
  (modify-syntax-entry ?\\ "\\" ldap-mode-syntax-table)
  (modify-syntax-entry ?* "." ldap-mode-syntax-table)
  (modify-syntax-entry ?+ "." ldap-mode-syntax-table)
  (modify-syntax-entry ?- "." ldap-mode-syntax-table)
  (modify-syntax-entry ?= "." ldap-mode-syntax-table)
  (modify-syntax-entry ?< "." ldap-mode-syntax-table)
  (modify-syntax-entry ?> "." ldap-mode-syntax-table))

;; --> font-lock-builtin-face
(defvar ldap-font-lock-keywords
  '(("%[^%]+%" 0 font-lock-comment-face t)
    ("\\<\\(reel\\|entier\\|booleen\\)\\>" 1 font-lock-type-face)
    ("\\<\\(var\\|\\|tantque\\|debut\\|fin\\|si\\|alors\\|sinon\\)\\>"
     1 ldap-keyword-face)
    ("\\<\\(saisir\\|afficher\\)\\>" 1 font-lock-keyword-face)
    ("\\(+\\|-\\|*\\|:=\\|>\\|<\\|>=\\|<=\\|/\\|=\\)" 1 font-lock-builtin-face)
    ("\\<\\(non\\|\\|et\\|ou\\)\\>" 1 font-lock-builtin-face)
    ("\\<\\(vrai\\|faux\\)\\>" 1 font-lock-constant-face)
    ("\\<\\([a-zA-Z][a-zA-Z_0-9]*\\)\\>" 1 font-lock-variable-name-face)))
;;;###autoload
(defun ldap-mode ()
  ""
  (interactive)
  (kill-all-local-variables)
  (local-set-key [menu-bar] (make-sparse-keymap))
  (setq major-mode 'ldap-mode)
  (setq mode-name "LDA")
  (setq local-abbrev-table ldap-mode-abbrev-table)
  (set-syntax-table ldap-mode-syntax-table)
  (make-local-variable 'paragraph-start)
  (setq paragraph-start (concat "$\\|" page-delimiter))
  (make-local-variable 'paragraph-separate)
  (setq paragraph-separate paragraph-start)
  (make-local-variable 'paragraph-ignore-fill-prefix)
  (setq paragraph-ignore-fill-prefix t)
  (make-local-variable 'indent-line-function)
  (setq indent-line-function 'ldap-indent)
  (make-local-variable 'require-final-newline)
  (setq require-final-newline t)
  (make-local-variable 'comment-start)
  (setq comment-start "{")
  (make-local-variable 'comment-end)
  (setq comment-end "}")
  (make-local-variable 'comment-column)
  (setq comment-column 2)
  (run-hooks 'ldap-mode-hook)
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(ldap-font-lock-keywords nil nil nil))
  (local-set-key "n" 'electric-ldap-fin)
  (run-hooks 'ldap-mode-hook))

(defun ldap-indent()
  (interactive)
  (let ((pt) (beg) (indent ldap-indent-level) (beg2))
    (setq pt (point))
    (beginning-of-line)
    (setq beg (point))
    (cond ((search-backward-regexp "[^ \t\n]" nil t)
	   (cond ((looking-at ";")
		  (setq indent (- indent ldap-indent-level))
		  (beginning-of-line)
		  (setq beg2 (point))
 		  (cond ((search-backward-regexp "[^ \t\n]" nil t)
 			 (beginning-of-line)
 			 (if (looking-at
			      "^[^\n]+\\(alors\\|sinon\\)[ \t]*$")
 			     (setq indent (- indent ldap-indent-level)))))
		  (goto-char beg2)))
	   (beginning-of-line)
	   (if (looking-at "^[^\n]*fin[;][ \t]*$")
	       (setq indent (- indent ldap-indent-level)))
	   (search-forward-regexp "[^ \t\n]" nil t)
	   (backward-char 1)
	   (setq indent (+ indent (current-column))))
	  (t
	   (setq indent (- indent ldap-indent-level))))
    (goto-char beg)
    (delete-horizontal-space)
    (if (looking-at "^[^\n]*fin[;.]?[ \t]*$")
	(setq indent (- indent ldap-indent-level)))
    (indent-to (max 0 indent))))

(defun electric-ldap-fin()
  (interactive)
  (insert "n")
  (save-excursion
    (beginning-of-line)
    (if (looking-at "^[ \t]*fin[ \t]*$")
	(ldap-indent))))
