open Printf

(* Types de Base *)

type typeBase = ValeurBooleene
              | ValeurEntiere
              | ValeurReel

(* Numero de la ligne pour l'affichage des erreurs *)		  

let line = ref 1

let erreur texte =
  Printf.fprintf stderr "%i: %s\n" !line texte;
  exit 1

(* Applique la methode print  une liste d'objet *)
let rec printall = function
    [] -> ()
  | obj :: reste ->
      obj#print;
      printall reste


(* Les classes pour la construction de l'arbre *)

class virtual expression (typ:typeBase) (reg:string) =
  object
    val t=typ
    val r=reg
    method typ=t
    method reg=r
    method typstr=
      match typ with
	ValeurBooleene | ValeurEntiere -> "int"
      | ValeurReel -> "float"
    method virtual print : unit
  end

class virtual instruction =
  object
    method virtual print : unit
  end

class variable typ reg (nom:string) =
  object
    inherit expression typ reg
    val n = nom
    method nom = n
    method print = printf "%s %s;\n"
	(match typ with
	  ValeurBooleene | ValeurEntiere -> "int"
        | ValeurReel                     -> "float"
        )
	reg
  end

class constante typ reg =
  object
    inherit expression typ reg
    method print = ()
  end

class evalV (var:variable) =
  object
    inherit expression var#typ var#reg
    method print = ()
  end

class tantque (label1:string) (label2:string)
              (cond:expression) (listeInstr:instruction list) =
  object
    inherit instruction
    val e = cond
    val l = listeInstr
    val lbl1 = label1
    val lbl2 = label2
    method print =
      printf "%s:\n" lbl1;
      e#print;
      printf "if (!%s) goto %s;\n" e#reg lbl2;
      printall l;
      printf "goto %s;\n" lbl1;
      printf "%s:\n" lbl2;
  end
      
class si (label1:string) (cond:expression) (listeInstr:instruction list) =
  object
    inherit instruction
    val e = cond
    val l = listeInstr
    val lbl1 = label1
    method print =
      e#print;
      printf "if (!%s) goto %s;\n" e#reg lbl1;
      printall l;
      printf "%s:\n" lbl1;
  end

class siAlors (label1:string) (label2:string) (cond:expression)
              (instr1:instruction list) (instr2:instruction list) =
  object
    inherit instruction
    val e = cond
    val l1 = instr1
    val l2 = instr2
    val lbl1 = label1
    val lbl2 = label2
    method print =
      e#print;
      printf "if (%s) goto %s;\n" e#reg lbl1;
      printall l2;
      printf "goto %s;\n" lbl2;      
      printf "%s:\n" lbl1;
      printall l1;
      printf "%s:\n" lbl2;
  end

class op_unaire (typ : typeBase) reg (op : string) (expr : expression) =
  object(self)
    inherit expression typ reg
    val aff = op
    method print = 
	expr#print ;
	printf "%s=(%s)%s%s;\n" reg self#typstr aff expr#reg
  end

class operation typ reg (op:string) (e1:expression) (e2:expression) =
  object(self)
    inherit expression typ reg
    val aff=op
    method print =
      e1#print;
      e2#print;
      printf "%s=(%s)%s%s%s;\n" reg self#typstr e1#reg aff e2#reg
  end

class affectation (var : variable)  (expr : expression) =
  object
    inherit instruction
    val e = expr 
    val v = var
    method print =
      e#print ;
      printf "%s=(%s)%s ;\n" v#reg v#typstr e#reg
    end

class afficherTxt (chaine:string) =
  object
    inherit instruction
    val texte = chaine
    method print = printf "printf(%s);\n" texte
  end

class afficher (expr:expression) =
  object
    inherit instruction
    val e = expr
    method print = printf "printf(\"%%%s\",%s);\n"
	(match e#typ with
	  ValeurReel -> "f"
	| ValeurEntiere -> "i"
	| ValeurBooleene -> "i"
	)
	e#reg
  end

class bloc (listeInstr:instruction list) =
  object
      inherit instruction
      val l = listeInstr
      method print = printall l
  end

class prog (p: bloc) =
  object
    val l=p
    method get = l
    method print =
      printf "\nint main(char *argv[], int argc) {\n";
      l#print ;
      printf "exit(0);\n" ;
      printf "}"
  end
